/// <reference types="node" />
import { AccountInfo, Connection, Keypair, PublicKey } from '@solana/web3.js';
export declare type PersistedAccountInfo = {
    pubkey: string;
    account: {
        lamports: number;
        data: [string, 'base64'];
        owner: string;
        executable: boolean;
        rentEpoch: number;
    };
};
export declare class AccountPersister {
    readonly targetDir: string;
    readonly connection?: Connection | undefined;
    constructor(targetDir: string, connection?: Connection | undefined);
    saveAccountInfo(address: PublicKey, accountInfo: AccountInfo<Buffer>, subdir?: string, label?: string): Promise<string>;
    saveAccount(address: PublicKey, connection?: Connection, data?: Buffer): Promise<string>;
    saveKeypair(id: string, keypair: Keypair, subdir?: string): Promise<string>;
    snapshot(snapshotLabel: string, addresses: string[], accountLabels: Record<string, string>, keypairs: Map<string, {
        keypair: Keypair;
        id: string;
    }>, maybeConnection?: Connection): Promise<string>;
    private _requireConnection;
}
export declare function loadAccount(address: PublicKey, sourceDir?: string, label?: string): Promise<PersistedAccountInfo>;
export declare function accountInfoFromPersisted(persisted: PersistedAccountInfo): {
    address: string;
    accountInfo: AccountInfo<Buffer>;
};
export declare function mapPersistedAccountInfos(persisteds: PersistedAccountInfo[]): Map<string, AccountInfo<Buffer>>;
