import { Keypair } from '@solana/web3.js';
import { RelayAccountState } from './types';
/** @private */
export declare type AmmanClient = {
    clearAddressLabels(): void;
    clearTransactions(): void;
    addAddressLabels(labels: Record<string, string>): Promise<void>;
    fetchAddressLabels(): Promise<Record<string, string>>;
    fetchAccountStates(address: string): Promise<RelayAccountState[]>;
    requestSnapshot(label?: string): Promise<string>;
    requestSaveAccount(address: string): Promise<string>;
    requestStoreKeypair(label: string, keypair: Keypair): Promise<void>;
    requestLoadKeypair(id: string): Promise<Keypair | undefined>;
    disconnect(): void;
    destroy(): void;
};
export declare type AmmanClientOpts = {
    autoUnref?: boolean;
    ack?: boolean;
};
/** @private */
export declare class ConnectedAmmanClient implements AmmanClient {
    readonly url: string;
    private readonly socket;
    private readonly ack;
    private constructor();
    private connect;
    clearAddressLabels(): void;
    clearTransactions(): void;
    addAddressLabels(labels: Record<string, string>): Promise<void>;
    fetchAddressLabels(): Promise<Record<string, string>>;
    fetchAccountStates(address: string): Promise<RelayAccountState[]>;
    requestSnapshot(label?: string): Promise<string>;
    requestSaveAccount(address: string): Promise<string>;
    requestStoreKeypair(id: string, keypair: Keypair): Promise<void>;
    requestLoadKeypair(id: string): Promise<Keypair | undefined>;
    /**
     * Disconnects this client and allows the app to shut down.
     * Only needed if you set `{ autoUnref: false }` for the opts.
     */
    disconnect(): void;
    /**
     * Disconnects this client preventing reconnects and allows the app to shut
     * down. Only needed if you set `{ autoUnref: false }` for the opts.
     */
    destroy(): void;
    private static _instance;
    static getInstance(url?: string, ammanClientOpts?: AmmanClientOpts): ConnectedAmmanClient;
}
/** @private */
export declare class DisconnectedAmmanClient implements AmmanClient {
    clearAddressLabels(): void;
    clearTransactions(): void;
    addAddressLabels(_labels: Record<string, string>): Promise<void>;
    fetchAddressLabels(): Promise<Record<string, string>>;
    fetchAccountStates(_address: string): Promise<RelayAccountState[]>;
    requestSnapshot(_label?: string): Promise<string>;
    requestSaveAccount(_address: string): Promise<string>;
    requestStoreKeypair(_label: string, _keypair: Keypair): Promise<void>;
    requestLoadKeypair(id: string): Promise<Keypair | undefined>;
    disconnect(): void;
    destroy(): void;
}
